/*------------------------------------------------------------------------------
 * MDK Middleware - Component ::USB:Device
 * Copyright (c) 2004-2019 Arm Limited (or its affiliates). All rights reserved.
 *------------------------------------------------------------------------------
 * Name:    RNDIS.c
 * Purpose: USB Device - RNDIS example
 *----------------------------------------------------------------------------*/

#include "rl_usb.h"                     /* RL-USB function prototypes         */

#include "Board_GLCD.h"
#include "GLCD_Config.h"

#include "SCU_LPC43xx.h"

extern GLCD_FONT GLCD_Font_6x8;
extern GLCD_FONT GLCD_Font_16x24;

// Main stack size must be multiple of 8 Bytes
#define APP_MAIN_STK_SZ (1024U)
uint64_t app_main_stk[APP_MAIN_STK_SZ / 8];
const osThreadAttr_t app_main_attr = {
  .stack_mem  = &app_main_stk[0],
  .stack_size = sizeof(app_main_stk)
};

/*------------------------------------------------------------------------------
 *        Application
 *----------------------------------------------------------------------------*/
__NO_RETURN void app_main (void *arg) {
  (void)arg;

  GLCD_SetBackgroundColor (GLCD_COLOR_BLUE);
  GLCD_SetForegroundColor (GLCD_COLOR_WHITE);
  GLCD_ClearScreen        ();
  GLCD_SetFont            (&GLCD_Font_16x24);
  GLCD_DrawString         (0U, 0U*24U, "      USB Device    ");
  GLCD_DrawString         (0U, 1U*24U, " CDC ACM Class RNDIS");
  GLCD_DrawString         (0U, 2U*24U, "   RNDIS Eth Bridge ");
  GLCD_DrawString         (0U, 4U*24U, "    USB_0 <-> ETH   ");
  GLCD_DrawString         (0U, 8U*24U, "  Keil Tools by ARM ");
  GLCD_DrawString         (0U, 9U*24U, "    www.keil.com    ");

  // Disable pull-ups on pins used for VBUS host drive signals P6_3 and P9_5
  SCU_PinConfigure        (6U, 3U, SCU_CFG_MODE_FUNC0 | SCU_PIN_CFG_PULLUP_DIS);
  SCU_PinConfigure        (9U, 5U, SCU_CFG_MODE_FUNC0 | SCU_PIN_CFG_PULLUP_DIS);

  USBD_Initialize         (0U);         // USB Device 0 Initialization
  USBD_Connect            (0U);         // USB Device 0 Connect

  for (;;) {                            // Loop forever
    osThreadFlagsWait     (0U, osFlagsWaitAny, osWaitForever);
  }
}
