/*------------------------------------------------------------------------------
 * Example main module
 * Copyright (c) 2019 Arm Limited (or its affiliates). All rights reserved.
 *------------------------------------------------------------------------------
 * Name:    main.c
 * Purpose: Main module
 *----------------------------------------------------------------------------*/

#include "cmsis_os2.h"                  // ::CMSIS:RTOS2

#include "LPC43xx.h"                    // Device header
#include "Board_LED.h"                  // ::Board Support:LED
#include "Board_Buttons.h"              // ::Board Support:Buttons
#include "GPIO_LPC43xx.h"               // Keil::Device:GPIO
#include "SCU_LPC43xx.h"                // Keil::Device:SCU


extern       void           app_main (void *arg);
extern const osThreadAttr_t app_main_attr;

/*------------------------------------------------------------------------------
 * main function
 *----------------------------------------------------------------------------*/
int main(void) {

  SystemCoreClockUpdate ();                     // Update System Core Clock info

  // LAN8720 on LPC430-Xplorer uses a GPIO for nRST
  GPIO_PortClock   (1);
  GPIO_SetDir      (0, 4, GPIO_DIR_OUTPUT);
  GPIO_PinWrite    (0, 4, 1);
  SCU_PinConfigure (1, 0, SCU_CFG_MODE_FUNC0);

  LED_Initialize ();                            // Initialize LED
  Buttons_Initialize ();                        // Initialize Buttons

  osKernelInitialize ();                        // Initialize CMSIS-RTOS2
  osThreadNew (app_main, NULL, &app_main_attr); // Create application main thread
  osKernelStart ();                             // Start thread execution

  for (;;) {}
}
